;****************************************************************************
;*                                                                          *
;*    P O D P R O G R A M Y    P R O    O V L A D A N I   D I S P L E J E   *
;*                                                                          *
;****************************************************************************        
; 
; Seznam podprogramu:
;
;    DISP_INIT
;    DISP_CLEAR
;    DISP_HEX  
;    DISP_DEC  
;    DISP_INT
;    DISP_SHOW
;
; Displej:
;
;   segmentovka 3    segmentovka 2    segmentovka 1   
;   --------------   --------------   -------------- 
;  |       a3     | |      a2      | |       a1     |
;  |      ----    | |      ----    | |       ----   |
;  |  f3 /g3 / b3 | |  f2 /g2 / b2 | |  f1 /g1 / b1 |
;  |     ----     | |     ----     | |     ----     |
;  | e3 /   / c3  | | e2 /   / c2  | | e1 /   / c1  |
;  |    ----      | |    ----      | |    ----      |
;  |     d3       | |     d2       | |     d1       |
;   --------------   --------------   -------------- 
;
;
; Komentar k podprogramum:
;
; === DISP_INIT ===
;
; inicializuje a zhasne displej. Tento podprogram staci pouzit 
; jen jednou na zacatku programu.
;
; pouziva registr: A
;
; === DISP_CLEAR ===
;
; zhasne vsechny segmenty na displeji.
;
; pouziva registr: A
;
; === DISP_HEX ===
; 
; zobrazi obsah registru DPTR v sestnackove soustave.
; Zobrazovany rozsah 0-FFFh.
; 
; pouziva registry: DPTR, A, R0, R1, R2, R3
; 
; === DISP_DEC === 
;
; zobrazi obsah registru DPTR v desikove soustave.
; Zobrazovany rozsah 0-999
; 
; pouziva registry: DPTR, A, B, R0, R1, R2, R3
;
; === DISP_INT ===
;
; zobrazi obsah registru A v desitkove soustave v rozsahu +/-99. 
; Zaporna cisla jsou uvazovana v doplnkovem kodu.
;
; pouziva registry: A, B, R0, R1, R2, R3
;   
; === DISP_SHOW ===
; 
; DISP_SHOW zobrazi na displeji znaky, jejichz tvar je ulozen v registrech
; R3, R2, R1. Prirazeni segmentu k jednotlivym bitum registru je nasledujici:
;
; pouziva: R0, R1, R2, R3, ACC
;
; Mapovani segmentu na jednotlive bity registru R1, R2 a R3.
; (1=segment sviti, 0=nesviti)
;
;         7    6    5    4    3    2    1    0
;  R1: | xx | g1 | f1 | e1 | d1 | c1 | b1 | a1 |
;  R2: | xx | g2 | f2 | e2 | d2 | c2 | b2 | a2 |
;  R3: | xx | g3 | f3 | e3 | d3 | c3 | b3 | a3 |
;
;
;  === KONSTANTY ZNAKU (urceno pro DISP_SHOW) ===
; 
;  Priklad pouziti:
;     ... 
;     mov R1, #char_r
;     mov R2, #char_r
;     mov R3, #char_E
;     lcall disp_show     ; zobrazi Err na displeji
;     ...
;
;
; KONSTANTY:
;
; cisla
;
char_0      equ 3fh
char_1      equ 06h
char_2      equ 5bh
char_3      equ 4fh
char_4      equ 66h
char_5      equ 6dh
char_6      equ 7dh
char_7      equ 07h
char_8      equ 7fh
char_9      equ 6fh 
;
; pismena
;
char_A      equ 77h
char_B      equ 7ch
char_C      equ 39h
char_D      equ 5eh
char_E      equ 79h
char_F      equ 71h
char_H      equ 76h
char_J      equ 1Fh
char_L      equ 38h
char_n      equ 54h
char_o      equ 5ch
char_P      equ 63h
char_r      equ 50h
char_S      equ 6dh
char_u      equ 1ch
char_Y      equ 6eh
;
; specialni znaky
;
char_blank  equ 00h	
char_minus  equ 40h
;
;
; 
; Podprogramy:
;
; DISP_INIT a DISP_CLEAR
;
disp_init:  clr  P1.0
            clr  P1.1
            mov  a,#36
disp_i1:    clr  P1.0
            setb P1.0
            djnz ACC,disp_i1
disp_clear: setb P1.1
            clr  P1.0
            setb P1.0
            clr  P1.1
            mov  a,#35
disp_i2:    clr  P1.0
            setb P1.0             
            djnz ACC,disp_i2
            ret
;
; DISP_HEX
;           
disp_hex:  mov  a, DPH
           mov  R0, DPL
           mov  dptr, #tabulka
           anl  a, #0Fh
           movc a, @a+dptr
           mov  R3, a
           mov  a, R0
           anl  a, #0F0h
           swap a
           movc a, @a+dptr
           mov  R2, a
           mov  a, R0
           anl  a, #0Fh
           movc a, @a+dptr
           mov  R1, a
           ljmp disp_show
;
; DISP_DEC
;           
disp_dec:  mov R0, DPL
           mov R1, DPH
           mov dptr, #tabulka
           mov R2, #0
disp_d1:   mov a, R0
           clr c
           subb a, #100
           mov R0, a
           mov  a, R1
           subb a, #0
           mov  R1, a
           inc  R2
           jnc  disp_d1
           mov  a, R2
           dec  a
           movc a, @a+dptr
           mov  R3, a
           mov  a, R0
           add  a, #100
           mov  b, #10
           div ab
           xch a, b
           movc a, @a+dptr
           mov R1, a
           mov a, b
           mov b, #10
           div ab
           xch a, b
           movc a, @a+dptr
           mov R2, a
           ljmp disp_show
;
; DISP_INT
;
disp_int:  mov dptr, #tabulka
           mov  R3, #char_blank
           jnb  acc.7, disp_plus
           mov  R3, #char_minus
           cpl  a
           inc  a
disp_plus: mov  b, a
           mov  R1, #char_minus
           mov  R2, #char_minus
           clr  c
           subb a, #64h
           jnc   disp_owf
           mov  a, b
           mov dptr, #tabulka
           mov b, #10
           div ab
           xch a, b
           movc a, @a+dptr
           mov R1, a
           mov a, b
           mov b, #10
           div ab
           xch a, b
           movc a, @a+dptr
           mov R2, a
disp_owf:  ljmp disp_show
;
; DISP_SHOW
;           
disp_show: setb P1.1
           clr  P1.0
           setb P1.0
           mov  R0,#1
           lcall disp_blank
           mov R0, #7
           mov  a, R3
           rlc  a
           mov R0, #7
           lcall disp_data
           mov  R0,#3
           lcall disp_blank
           mov  R0, #4
           mov  a, R2
           rlc  a
           lcall disp_data
           mov  R0, #4
           lcall disp_blank
           mov  R0, #3
           lcall disp_data
           mov  R0, #3
           mov  a, R1
           rlc  a
           lcall disp_data
           mov  R0, #1
           lcall disp_blank
           mov  R0, #1
           lcall disp_data
           mov  R0, #2
           lcall disp_blank
           mov  R0, #3
           lcall disp_data
           mov  R0, #3
           lcall disp_blank
           ret
           nop      ; 2x nop nutne pro ladeni (brekpoint je dlouhy 3 byte)
           nop
;
;
; Pomocne podprogramy
;
; DISP_DATA
;
; A - data; R0 - pocet vysilanych bitu
;
disp_data: rlc  a
           mov  P1.1, c
           clr  P1.0
           setb P1.0
           djnz R0, disp_data
           ret
           nop
           nop
;
; DISP_BLANK
;
; R0 - delka
;
disp_blank: clr  P1.1                                                         
disp_b1:    clr  P1.0
            setb P1.0
            djnz R0, disp_b1
            ret
            nop
            nop
;            
;
Tabulka:    db char_0, char_1, char_2, char_3, char_4
            db char_5, char_6, char_7, char_8, char_9
            db char_A, char_B, char_C, char_D, char_E
            db char_F
;
;***************************************************************************
