$nomod51

;*****************************************************
;*                                                   *
;*   SRTL51_S.A51    Version 1.2Y                    *
;*                                                   *
;*   Simple Real Time Library for 8051, small model  *
;*                                                   *
;*   Copyright  Libor Hlavac, DiCom  (c)  1994       *
;*                                                   *
;*   Last edited:  27.11.1994  00:00                 *
;*                                                   *
;*****************************************************

		name 	srtl51_s

$include 	(reg31.inc)          	; definice 8051, 8031 registru


;

					; dulezite konstanty a makra

%DEFINE	(DS51) 	    (1)			; pouzivame DS51 ??
%DEFINE (NUM_TASKS) (3)                 ; pocet uloh (max. 3 !!!)
%DEFINE (NUM_SIG)   (3)			; pocet signalu (omezen pouze
					;  kapacitou vnitrni pameti)

MIN_STACK_SIZE	equ	18h		; minimalni pozadovana velikost
					; zasobniku ***

NUM_TASKS	equ	%NUM_TASKS
NUM_SIG		equ	%NUM_SIG

;

					; ostatni konstanty
TIME_DIV	equ	1000		; casova konstanta pro preruseni
					; 1 ms
BAUD_9600	equ	3		; casova konstanta pro generovani
					; prenosove rychlosti serioveho portu
					; => 9600 bit/s
RT_BANK		equ	18h		; RT knihovna pouziva registrovou
					; banku 3 a toto je jeji pocatecni
					; psw

;

		dseg at	21h
					; globalni data
rt_slice:	ds	1		; interval prepinani kontextu (ms)
rt_slice_timer:	ds	1		; citac intervalu prepinani kontextu
rt_running:	ds	1		; ID prave aktivni ulohy

		bseg at	0h

rt_delayb:	dbit	1		; minimalne jedna uloha je pozdrzena
rt_released:	dbit	1		; minimalne jedna uloha byla uvolnena

;

					; data pro jednotlive ulohy
		dseg at	24h

rt_task_stat:	ds	NUM_TASKS       ; stavy jednotlivych uloh
rt_task_delay:	ds	NUM_TASKS	; citace pro pozdrzeni
rt_task_sig:	ds	NUM_TASKS	; cislo ulohy, ktera nasleduje
					; ve fronte signalu
rt_task_stack:	ds	NUM_TASKS	; aktualni hodnota zasobniku


		dseg at	33h
?rt51_stacks:
					; zasobniky pro ulohy
%REPEAT (%NUM_TASKS)
		(
		ds	MIN_STACK_SIZE
		)

					; definice stavu ulohy:
TS_FREE		equ	0		; neni pouzita
TS_READY	equ	1		; pripravena
TS_NOT_READY	equ	2		; nepripravena (cekani na signal)
TS_DELAY	equ	3		; pozdrzena

;

					; data pro jednotlive signaly
		dseg at	30h

rt_sig_stat:	ds	NUM_SIG		; stav signalu ci ID prvniho tasku
					; ve fronte

SIG_OPENED	equ	0FEh		; signal otevren
SIG_CLOSED	equ	0FFh		; signal uzavren

;

		cseg	at 8000h
		clr	EA
		ljmp  	cold_start  	; studeny start systemu

;

		cseg	at 800Bh
		ljmp	timer0_int  	; preruseni od casovace

;

		using	0               ; implicitni uloha 0 pracuje
					; s registrovou bankou 0

		cseg at 09800h   	; kod jadra zacina zde

cold_start:
     		clr	a               ; vynulovani obsahu vnitrni pameti
		mov	psw, a
		mov	r0, #0FFh
data_init:	mov	@r0, a
		djnz	r0, data_init

;
;  This part of startup initializes processor.
;

hard_init:

		mov	PCON,  #00000000b
;				    IDL  idle mode
;				    PD   power-down mode
;				    GF0  general purpose flag
;				    GF1  general purpose flag
;				 SMOD baud rate doubling

		orl	TCON,  #00010000b
;				 IT0  INT0 trigger 0 - low level
;                                                     1 - falling edge
;				 IE0  INT0 edge flag
;				 IT1  INT1 trigger 
;				 IE1  INT1 edge flag
;				 TR0  timer 0 run control bit
;				 TF0  timer 0 overflow flag
;				 TR1  timer 1 run control bit
;				 TF1  timer 1 overflow flag

		orl	TMOD,  #00000001b
;				 timer 0 mode 00 - 8 bit,prescaler
;                                                  01 - 16 bit
;                                                  10 - 8 bit,auto-reload
;                                                  11 - 2 x 8 bit
;				  timer 0 C/T# (counter / timer)
;                                                   0  - timer
;                                                   1  - counter
;				  timer 0 GATE gating control
;				                     1  - gating on
;				                     0  - gating off
;				  timer 1 mode 
;				 timer 1 C/T# 
;				 timer 1 GATE 

;

		mov	TL0,   #LOW (-TIME_DIV)
		mov	TH0,   #HIGH(-TIME_DIV)

;		mov	TL1,   #(-BAUD_9600)
;		mov	TH1,   #(-BAUD_9600)

;		mov	SCON,  #01011010b
;				 RI   receiver interrupt
;				  TI   trasmitter interrupt
;				  RB8  receiver bit 8
;				  TB8  trasmitter bit 8
;				  REN  receiver enable
;				  SM2  multiprocessor feature
;				  SM1  serial mode
;				           SM0   00 - shift register, fixed
;						 01 - 8-bit UART, variable
;						 10 - 9-bit UART, fixed
;						 11 - 9-bit UART, variable

		orl     IE,    #00000010b
;				   EX0  external INT0
;				   ET0  timer 0 interrupt
;				   EX1  external INT1
;				   ET1  timer 1 interrupt
;				   ES   serial interrupt
;				 EA   all interrupts

		mov     IP,    #00000000b
;				    PX0  external INT0 level
;				    PT0  timer 0 interrupt level
;				    PX1  external INT1 level
;				    PT1  timer 1 interrupt level
;				    PS   serial interrupt level

;
;  Priority within level:
;
;   IE0           	        High
;   TF0                        
;   IE1                          
;   TF1                        
;   RI + TI                     Low
;

		mov	PSW,   #00000000b
;				  P     parity flag
;				  OV    overflow flag
;				  RS0/1 register bank select
;				 F0    user flag 0
;				 AC    auxiliary carry flag
;				 CY    carry flag


		mov	sp, #?rt51_stacks - 1	; nastaveni zasobniku ulohy 0
		mov	rt_task_stat, #TS_READY	; uloha 0 pripravena
		mov	rt_running, #0		; aktualni uloha => 0
		mov	rt_slice, #10           ; interval pro task switching
		mov	rt_slice_timer, #10     ; => 10 ms

		setb	EA                      ; povoleni preruseni

		ljmp	main

main		equ	8100h



;

; funkce RT knihovny

;

		org	983dh

					; definice navratovych kodu
RT_OK		equ	0               ; vse v poradku
RT_INVALID_TASK	equ	1		; nespravne cislo ulohy
RT_TASK_ACTIVE	equ	2		; uloha je jiz aktivni

; funkce rt_create
; - vytvori ulohu pod cislem r7 (0-2) z programu na adrese r4 r5.

rt_create:      mov	c, EA           ; uloz stav EA
		push	psw
		push	acc             ; uloz registry
		clr	EA              ; parametry na zasobnik
		mov	a, r7
		push	acc
		mov	a, r5
		push	acc
		mov	a, r4
		push	acc
		mov	psw, #RT_BANK	; registry RT knihovny
		pop	acc             ; parametry zpet do registru
		mov	r4, a
		pop	acc
		mov	r5, a
		pop	acc
		mov	r6, a
		cjne	a, #NUM_TASKS, $+3    	; test na neplatne cislo ulohy
		mov	r7, #RT_INVALID_TASK
		jnc     rt_create_end
		add	a, #rt_task_stat
		mov	r0, a
		mov	a, @r0			; test neni-li uloha jiz
		mov	r7, #RT_TASK_ACTIVE     ; aktivni
		jnz	rt_create_end
		mov	@r0, #TS_READY          ; uloha bude pripravena
		mov	a, r0
		add	a, #(3 * NUM_TASKS)     ; posun se na ukazatel
		mov	r0, a                   ; zasobniku
		mov	a, #-MIN_STACK_SIZE     ; vypocti zacatek zasobniku
		mov	r2, #0FFh		; a registrovou banku
		inc	r6
stack_sum_loop:	add	a, #MIN_STACK_SIZE
		inc	r2
		djnz	r6, stack_sum_loop
		add	a, #?rt51_stacks
		mov	r1, a
		mov	a, r5                   ; vlozeni startovni adresy
		mov	@r1, a			; na zasobnik
		inc	r1
		mov	a, r4
		mov	@r1, a
		inc	r1
		mov	a, r2
		rl	a
		rl	a
		rl	a
		mov	@r1, a                  ; ulozeni psw ulohy
		mov	a, r1                   ; fiktivni ulozeni dalsich
		add	a, #4			; registru
		mov	@r0, a                  ; ulozeni stavu zasobniku
		mov	r7, #RT_OK

rt_create_end:  pop	acc                     ; obnova registru
		pop	psw

		mov	EA, c                   ; a preruseni
		ret				; konec

;
					; definice navratovych kodu
RT_TASK_FREE	equ	3		; uloha je jiz uvolnena


; funkce rt_kill
; - zrusi vykonavani ulohy r7

rt_kill:       mov	c, EA
		push	psw     	; ulozeni psw a acc
		push	acc
		clr	EA
		mov	a, r7
		mov	psw, #RT_BANK   	; nastaveni banky RT knihovny
		cjne	a, #NUM_TASKS, $+3
		mov	r7, #RT_INVALID_TASK    ; kontrola cisla ulohy (0-2)
		jnc     rt_kill_end
		add	a, #rt_task_stat
		mov	r0, a
		mov	a, @r0                  ; jiz zrusena ci nepouzita
		mov	r7, #RT_TASK_FREE	; uloh anelze zrusit
		jz	rt_kill_end
		mov	@r0, #TS_FREE           ; vlastni zruseni
		mov	r7, #RT_OK

rt_kill_end:    pop	acc                  	; obnova registru
		pop	psw
		mov	EA, c
		ret

;

; funkce rt_delay
; - pozdrzi vykonavani ulohy o r7 ms

rt_delay:	push	psw         		; uschova
		push	acc
		clr	EA
		mov	a, r7
		mov	psw, #RT_BANK    	; nastaveni banky RT knihovny
		mov	r7, a
		mov	a, rt_running
		mov	r0, #rt_task_stat
		add	a, r0
		mov	r0, a			; prave aktvni uloha bude
		mov	@r0, #TS_DELAY          ; pozdrzena
		add	a, #NUM_TASKS
		mov	r0, a
		mov	a, r7			; nastav citac na parametr
		mov	@r0, a                  ; r7
		setb	rt_delayb               ; nekdo je pozdrzen
		ljmp	rt_task_switch          ; proved prepnuti kontextu

;
					; definice navratovych kodu
RT_INVALID_SIG	equ	1		; nespravne cislo signalu

; funkce rt_init_sig
; - pocatecni nastaveni signalu
; - nastavi signal r7 na hodnotu podle r5 (1 = otevren, 0 = zavren)
;   a nuluje jeho frontu

rt_init_sig:	mov	c, EA
		push	psw
		push	acc
		clr	EA     		; vzajemne vylouceni operaci nad
		mov	a, r5		; signaly
		push	acc
		mov	a, r7
		mov	psw, #RT_BANK
		mov	r7, a
		pop	acc
		mov	r5, a
		mov	a, r7
		cjne	a, #NUM_SIG, $+3
		mov	r7, #RT_INVALID_SIG 	; kontrola cisla signalu
		jnc	rt_init_end
		add	a, #rt_sig_stat
		mov	r0, a
		mov	a, r5           	; podle r5 zapis stav
		mov	@r0, #SIG_CLOSED
		jz	rt_init_end
		dec	@r0
rt_init_end:    pop	acc
		pop	psw
		mov	EA, c                   ; konec
		ret

;

; funkce rt_wait_sig
; - provede wait na signal r7

rt_wait_sig:   mov	c, EA
		push	psw
		push	acc
		clr	EA
		mov	a, r7
		mov	psw, #RT_BANK
		cjne	a, #NUM_SIG, $+3
		mov	r7, #RT_INVALID_SIG	; kontrola cisla signalu
		jnc	rt_wait_end
		add	a, #rt_sig_stat         ; testuje se stav signalu
		mov	r0, a
		mov	a, @r0                  ; je-li otevren
		cjne	a, #SIG_OPENED, sig_busy
		mov	@r0, #SIG_CLOSED        ; tak se jen zavre
rt_wait_end:    pop	acc
		pop	psw
		mov	EA, c
		ret

sig_busy:	mov	r1, #rt_task_stat	; jinak se aktivni uloha
		mov	a, rt_running
		add	a, r1
		mov	r1, a
		mov	@r1, #TS_NOT_READY      ; oznaci jako nepripravena
		mov	a, #rt_task_sig         ; a zaradi se na konec
		add	a, rt_running           ; fronty
		mov	r1, a
		mov	@r1, #SIG_CLOSED        ; oznacuje konec fronty
		mov	a, @r0
		cjne	a, #SIG_CLOSED, sig_next ; staci zapsat do signalu
		mov     a, rt_running           ; na prvni misto fronty
		mov	@r0, a
		ljmp	rt_task_switch          ; prepnuti kontextu

sig_next:       mov	a, @r0                  ; je treba projit frontu
		add	a, #rt_task_sig         ; a zapsat se na konec
		mov     r0, a
		mov	a, @r0
		cjne	a, #0FFh, sig_next      ; hledej konec fronty
		mov	a, rt_running
		mov	@r0, a                  ; zapis tak tuto ulohu
		ljmp	rt_task_switch          ; prepnuti kontextu

;

; funkce rt_send_sig
; - provede funkci send nad signalem r7

rt_send_sig:	mov	c, EA
		push	psw
		push	acc
		clr	EA              	; zakazane preruseni pro
		mov	a, r7			; vzajemne vylouceni operaci
		mov	psw, #RT_BANK           ; nad signaly
		cjne	a, #NUM_SIG, $+3
		mov	r7, #RT_INVALID_SIG     ; kontrola cisla signalu
		jnc	rt_wait_end
		add	a, #rt_sig_stat
		mov	r0, a
		mov	a, @r0                  ; jestlize je jiz signal
		cjne	a, #SIG_OPENED, not_free  ; otevren tak se nic nedeje
		sjmp	rt_send_end
not_free:	cjne	a, #SIG_CLOSED, sig_free  ; jestlize je uzavren ale
		mov	@r0, #SIG_OPENED          ; nikdo neceka tak jen
rt_send_end:    pop	acc                       ; otevri
		pop	psw
		mov	EA, c
		ret
sig_free:	mov	a, @r0              	; jinak prvniho ve fronte
		add	a, #rt_task_stat
		mov	r1, a
		mov	@r1, #TS_READY          ; oznacime pripraveny
		mov	a, @r0
		add	a, #rt_task_sig
		mov	r1, a
		mov	a, @r1
		mov	@r0, a                  ; a vypustime z fronty
		sjmp	rt_send_end

;

; funkce rt_set_slice
; - nastaveni intervalu prepinani kontextu na r7 ms

rt_set_slice:  mov	rt_slice, r7
		ret

;

; preruseni od casovace 0 pouziva knihovna pro vlastni ucely
; - prerusuje se kazdou 1 ms
; - casovani pozdrzeni
; - prepinani kontextu po vyprseni nastaveneho intervalu


		using	3

timer0_int:     lcall	rt_int_ret	; nejprve proved RETI, aby se dala
					; cast techto rutin volat nejen
					; z preruseni

		push	psw
		push	acc             ; uschova psw, acc
		clr	EA

		clr	TR0             ; dorovnani casovace na spravnou
		mov	a, TL0		; hodnotu
		add	a, #(LOW(-TIME_DIV) + 7)
		mov     TL0, a
		mov	a, TH0
		addc	a, #HIGH(-TIME_DIV)
		mov	TH0, a
		setb	TR0

		mov	psw, #RT_BANK 	; prepnuti na banku knihovny

		jnb	rt_delayb, no_delay	; jeslize je nekdo pozdrzeny
						; tak se musi upravit citace
		clr	rt_delayb
		mov	r0, #rt_task_stat
		mov	r1, #rt_task_delay

delay_loop:     mov	a, @r0                  ; jestlize je uloha pozdrzena
		cjne	a, #TS_DELAY, not_delay
		dec	@r1                     ; zmensi citac
		mov	a, @r1
		jz	released                ; a pripadne uvolni
		setb	rt_delayb
		sjmp	not_delay

released:       setb	rt_released             ; nekdo byl uvolnen
		mov	@r0, #TS_READY
not_delay:	inc	r0
		inc	r1
		cjne	r0, #(rt_task_stat + NUM_TASKS), delay_loop

no_delay:	dec	rt_slice_timer      	; uz je cas na prepnuti
		mov	a, rt_slice_timer	; kontextu ?
		jz	rt_to_switch

		jnb	rt_released, rt_no_switch ; kdyz byl nekdo uvolnen
		mov	a, rt_running             ; a neni aktivni zadna
		inc	a                         ; uloha tak se prepina
		jz	rt_to_switch		  ; vzdy !!!!!

rt_no_switch:	pop	acc
		pop	psw
		setb	EA
		ret

rt_to_switch: 	mov	a, rt_slice		; znovunastaveni citace
		mov	rt_slice_timer, a

rt_task_switch: mov	a, rt_running          ; bezela nejaka uloha ?
		inc	a
		jnz	no_nothing
		pop	acc                    ; ne, zahod registry na
		pop	acc	               ; zasobniku
		pop	acc
		pop	acc
		clr	a
		sjmp	rt_schedule2

no_nothing:     dec	a                   	; nejprve ulozime
		push	dpl                     ; stary kontext
		push	dph
		push    b
		mov	r0, #rt_task_stack
		add	a, r0
		mov	r0, a
		mov	@r0, sp                 ; takze i zasobnik

rt_schedule:	mov	a, rt_running           ; budeme hledat novy proces
rt_schedule2:	mov	r0, #rt_task_stat       ; prvni v rade, ktery je
		add	a, r0                   ; pripraven
		mov	r0, a
		mov	r1, a

schedule_loop:  inc	r0
		cjne	r0, #(rt_task_stat + NUM_TASKS), not_after_last
		mov	r0, #rt_task_stat
not_after_last:	mov	a, @r0
		cjne    a, #TS_READY, not_ready
		clr	c
		mov	a, r0
		subb	a, #rt_task_stat

rt_switch:	mov	rt_running, a       	; nasli jsme ho
		mov	r0, #rt_task_stack      ; nacti kontext
		add	a, r0
		mov	r0, a
		mov	sp, @r0
		pop	b
		pop	dph
		pop	dpl
		pop     acc
		pop	psw
		setb	EA
		ret                             ; a navrat do nove ulohy

not_ready:      mov	a, r0                   ; zadny pripraveny neni
		xrl	a, r1
		jnz	schedule_loop

		mov	rt_running, #-1		; nebezi nic !

		setb	EA
		ljmp	$			; tady budeme cekat na
						; dalsi preruseni


rt_int_ret:	reti

		end